@@ -25,10 +25,10 @@ from urllib.parse import urlparse
 from gettext import gettext as _
 
 import gi
-from gi.repository import Gst, Gtk, GObject, Gio
+from gi.repository import Gst, Gtk, GLib, GObject, Gio
 
-from soundconverter.fileoperations import vfs_encode_filename, file_encode_filename
-from soundconverter.fileoperations import unquote_filename, vfs_makedirs, vfs_unlink
+from soundconverter.fileoperations import vfs_encode_filename
+from soundconverter.fileoperations import unquote_filename, vfs_unlink
 from soundconverter.fileoperations import vfs_rename
 from soundconverter.fileoperations import vfs_exists
 from soundconverter.fileoperations import beautify_uri
@@ -67,6 +67,7 @@ audio_profiles_list = []
 audio_profiles_dict = {}
 
 try:
+    gi.require_version('GConf', '2.0')
     from gi.repository import GConf
     _GCONF = GConf.Client.get_default()
     profiles = _GCONF.all_dirs(_GCONF_PROFILE_LIST_PATH)
@@ -87,7 +88,7 @@ try:
             profile = description, extension, pipeline
             audio_profiles_list.append(profile)
             audio_profiles_dict[description] = profile
-except ImportError:
+except (ImportError, ValueError):
     pass
 
 required_elements = ('decodebin', 'fakesink', 'audioconvert', 'typefind', 'audiorate')
@@ -99,40 +100,50 @@ for element in required_elements:
 gstreamer_source = 'giosrc'
 gstreamer_sink = 'giosink'
 encode_filename = vfs_encode_filename
-print('  using gio')
 
 # used to dismiss codec installation if the user already canceled it
 user_canceled_codec_installation = False
 
 encoders = (
-    ('flacenc', 'FLAC'),
-    ('wavenc', 'WAV'),
-    ('vorbisenc', 'Ogg Vorbis'),
-    ('oggmux', 'Ogg Vorbis'),
-    ('id3mux', 'MP3 tags'),
-    ('xingmux', 'VBR tags'),
-    ('lamemp3enc', 'MP3'),
-    ('faac', 'AAC'),
-    ('mp4mux', 'AAC'),
-    ('opusenc', 'Opus'),
+    ('flacenc', 'FLAC', 'flac-enc'),
+    ('wavenc', 'WAV', 'wav-enc'),
+    ('vorbisenc', 'Ogg Vorbis', 'vorbis-enc'),
+    ('oggmux', 'Ogg Vorbis', 'vorbis-mux'),
+    ('id3mux', 'MP3 tags', 'mp3-id-tags'),
+    ('id3v2mux', 'MP3 tags', 'mp3-id-tags'),
+    ('xingmux', 'VBR tags', 'mp3-vbr-tags'),
+    ('lamemp3enc', 'MP3', 'mp3-enc'),
+    ('faac', 'AAC', 'aac-enc'),
+    ('avenc_aac', 'AAC', 'aac-enc'),
+    ('mp4mux', 'AAC', 'aac-mux'),
+    ('opusenc', 'Opus', 'opus-enc'),
 )
 
 available_elements = set()
+functions = dict()
 
-for encoder, name in encoders:
+for encoder, name, function in encoders:
     have_it = bool(Gst.ElementFactory.find(encoder))
     if have_it:
         available_elements.add(encoder)
     else:
-        print(('  "%s" gstreamer element not found'
-               ', disabling %s output.' % (encoder, name)))
+        print('  %s gstreamer element not found' % encoder)
+    function += '_' + name
+    functions[function] = functions.get(function) or have_it
+
+for function in sorted(functions):
+    if not functions[function]:
+        print('  disabling %s output.' % function.split('_')[1])
 
 if 'oggmux' not in available_elements:
     available_elements.discard('vorbisenc')
+if 'mp4mux' not in available_elements:
+    available_elements.discard('faac')
+    available_elements.discard('avenc_aac')
 
 
 class Pipeline(BackgroundTask):
-    """A background task for running a GstPipeline."""
+    """ A background task for running a GstPipeline. """
 
     def __init__(self):
         BackgroundTask.__init__(self)
@@ -202,7 +213,7 @@ class Pipeline(BackgroundTask):
             self.done()
             return
         self.done()
-        show_error('Error', 'failed to install plugins: %s' % GObject.markup_escape_text(str(result)))
+        show_error('Error', 'failed to install plugins: %s' % GLib.markup_escape_text(str(result)))
 
     def on_error(self, error):
         self.error = error
@@ -268,7 +279,7 @@ class Pipeline(BackgroundTask):
 
                 self.parsed = True
 
-            except GObject.GError as e:
+            except GLib.GError as e:
                 show_error('GStreamer error when creating pipeline', str(e))
                 self.error = str(e)
                 self.eos = True
@@ -292,11 +303,9 @@ class Pipeline(BackgroundTask):
 
     def get_position(self):
         return NotImplementedError
-        
+
     def query_duration(self):
-        """
-        Ask for the duration of the current pipeline.
-        """
+        """ Ask for the duration of the current pipeline. """
         try:
             if not self.sound_file.duration and self.pipeline:
                 self.sound_file.duration = self.pipeline.query_duration(Gst.Format.TIME)[1] / Gst.SECOND
@@ -305,8 +314,9 @@ class Pipeline(BackgroundTask):
         except Gst.QueryError:
             self.sound_file.duration = None
 
+
 class TypeFinder(Pipeline):
-    def __init__(self, sound_file):
+    def __init__(self, sound_file, silent=False):
         Pipeline.__init__(self)
         self.sound_file = sound_file
 
@@ -318,10 +328,22 @@ class TypeFinder(Pipeline):
         # decodebin. we can't use our own typefind before decodebin anymore,
         # since its caps would've been the same as decodebin's sink caps.
         self.add_signal('typefind', 'have-type', self.have_type)
+        self.silent = silent
+
+    def log(self, *args):
+        """ the output of TypeFinder can be disabled either with
+        the -q command line option or by setting self.silent to
+        True """
+        if not self.silent:
+            log(*args)
+
+    def on_error(self, error):
+        self.error = error
+        self.log('ignored-error: %s (%s)' % (error, ' ! '.join(self.command)))
 
     def set_found_type_hook(self, found_type_hook):
         self.found_type_hook = found_type_hook
-    
+
     def pad_added(self, decoder, pad):
         """ called when a decoded pad is created """
         self.query_duration()
@@ -335,12 +357,12 @@ class TypeFinder(Pipeline):
             if t in mime_type:
                 self.sound_file.mime_type = mime_type
         if not self.sound_file.mime_type:
-            log('mime type skipped: %s' % mime_type)
+            self.log('mime type skipped: %s' % mime_type)
         for t in filename_blacklist:
             if fnmatch(self.sound_file.uri, t):
                 self.sound_file.mime_type = None
-                log('filename blacklisted (%s): %s' % (t, self.sound_file.filename_for_display))
-        
+                self.log('filename blacklisted (%s): %s' % (t, self.sound_file.filename_for_display))
+
         return True
 
     def finished(self):
@@ -353,7 +375,7 @@ class TypeFinder(Pipeline):
 
 
 class Decoder(Pipeline):
-    """A GstPipeline background task that decodes data and finds tags."""
+    """ A GstPipeline background task that decodes data and finds tags. """
 
     def __init__(self, sound_file):
         Pipeline.__init__(self)
@@ -370,9 +392,7 @@ class Decoder(Pipeline):
         pass
 
     def query_position(self):
-        """
-        Ask for the stream position of the current pipeline.
-        """
+        """ Ask for the stream position of the current pipeline. """
         try:
             if self.pipeline:
                 self.position = max(0, self.pipeline.query_position(
@@ -381,9 +401,7 @@ class Decoder(Pipeline):
             self.position = 0
 
     def found_tag(self, decoder, something, taglist):
-        """
-        Called when the decoder reads a tag.
-        """
+        """ Called when the decoder reads a tag. """
         debug('found_tags:', self.sound_file.filename_for_display)
         taglist.foreach(self.append_tag, None)
 
@@ -399,8 +417,8 @@ class Decoder(Pipeline):
             'datetime',
             'year',
             'timestamp',
-            'disc-number',
-            'disc-count',
+            'album-disc-number',
+            'album-disc-count',
         )
         if tag not in tag_whitelist:
             return
@@ -452,7 +470,7 @@ class Decoder(Pipeline):
 
 
 class TagReader(Decoder):
-    """A GstPipeline background task for finding meta tags in a file."""
+    """ A GstPipeline background task for finding meta tags in a file. """
 
     def __init__(self, sound_file):
         Decoder.__init__(self, sound_file)
@@ -478,15 +496,15 @@ class TagReader(Decoder):
         Pipeline.finished(self)
         self.sound_file.tags_read = True
         if self.found_tag_hook:
-            GObject.idle_add(self.found_tag_hook, self)
+            GLib.idle_add(self.found_tag_hook, self)
 
 
 class Converter(Decoder):
-    """A background task for converting files to another format."""
+    """ A background task for converting files to another format. """
 
     def __init__(self, sound_file, output_filename, output_type,
                  delete_original=False, output_resample=False,
-                 resample_rate=48000, force_mono=False):
+                 resample_rate=48000, force_mono=False, ignore_errors=False):
         Decoder.__init__(self, sound_file)
 
         self.output_filename = output_filename
@@ -506,6 +524,8 @@ class Converter(Decoder):
         self.overwrite = False
         self.delete_original = delete_original
 
+        self.ignore_errors = ignore_errors
+
         self.got_duration = False
 
     def init(self):
@@ -571,9 +591,13 @@ class Converter(Decoder):
                 log('Cannot remove \'%s\'' % beautify_uri(self.output_filename))
 
     def on_error(self, error):
-        Pipeline.on_error(self, error)
-        show_error('<b>%s</b>' % _('GStreamer Error:'),
-                   '%s\n<i>(%s)</i>' % (error, self.sound_file.filename_for_display))
+        if self.ignore_errors:
+            self.error = error
+            log('ignored-error: %s (%s)' % (error, ' ! '.join(self.command)))
+        else:
+            Pipeline.on_error(self, error)
+            show_error('<b>%s</b>' % _('GStreamer Error:'),
+                    '%s\n<i>(%s)</i>' % (error, self.sound_file.filename_for_display))
 
     def set_vorbis_quality(self, quality):
         self.vorbis_quality = quality
@@ -634,11 +658,15 @@ class Converter(Decoder):
         if 'id3mux' in available_elements:
             # add tags
             cmd += '! id3mux '
+        elif 'id3v2mux' in available_elements:
+            # add tags
+            cmd += '! id3v2mux '
 
         return cmd
 
     def add_aac_encoder(self):
-        return 'faac bitrate=%s ! mp4mux' % (self.aac_quality * 1000)
+        encoder = 'faac' if 'faac' in available_elements else 'avenc_aac'
+        return '%s bitrate=%s ! mp4mux' % (encoder, self.aac_quality * 1000)
 
     def add_opus_encoder(self):
         return 'opusenc bitrate=%s bitrate-type=vbr bandwidth=auto ! oggmux' % (self.opus_quality * 1000)
@@ -649,7 +677,7 @@ class Converter(Decoder):
 
 
 class ConverterQueue(TaskQueue):
-    """Background task for converting many files."""
+    """ Background task for converting many files. """
 
     def __init__(self, window):
         TaskQueue.__init__(self)
@@ -751,7 +779,7 @@ class ConverterQueue(TaskQueue):
         if duration:
             self.duration_processed += duration
 
-        # rename temporary file 
+        # rename temporary file
         newname = self.window.prefs.generate_filename(task.sound_file)
         debug(beautify_uri(task.output_filename), '->', beautify_uri(newname))
 
@@ -804,7 +832,7 @@ class ConverterQueue(TaskQueue):
         TaskQueue.abort(self)
         self.window.set_sensitive()
         self.reset_counters()
-    
+
     def start(self):
-        #self.waiting_tasks.sort(key=Converter.get_duration,reverse=True)
+        #self.waiting_tasks.sort(key=Converter.get_duration, reverse=True)
         TaskQueue.start(self)
